import os
import logging
from kubernetes import config, client

__author__ = """Marco Masetti"""
__email__ = 'marco.masetti@sky.uk'
__version__ = '0.6.2'

log = logging.getLogger(__name__)


def config_client_old():
    if os.getenv('KUBERNETES_SERVICE_HOST'):
        try:
            print("Connecting using incluster config...")
            log.debug("Connecting using incluster config...")
            config.load_incluster_config()
            log.debug("Connected...")
        except Exception as e:
            print("Error connecting to local k8s cluster: {}".format(e))
            log.error("Error connecting to local k8s cluster: {}".format(e))
    else:
        try:
            print("Connecting via kube config file...")
            log.debug("Connecting via kube config file...")
            config.load_kube_config()
            log.debug("Connected...")
        except Exception as e:
            print("Error connecting to k8s: {}".format(e))
            log.error("Error connecting to k8s: {}".format(e))


def has_local_config_file():
    config_path = os.path.expanduser(
        os.environ.get('KUBECONFIG', '~/.kube/config'))
    return os.path.exists(config_path)


def config_client():
    """
    Determines how to configure k8s client:
    1. From a local configuration file if it exists (`~/.kube/config`). You
       can specify which context you want to use as well through the
       `KUBERNETES_CONTEXT` key in the environment.
    2. From the cluster configuration if executed from a Kubernetes pod and
       the CHAOSTOOLKIT_IN_POD is set to `"true"`.
    3. From a mix of the following environment keys:

        * KUBERNETES_HOST: Kubernetes API address

        You can authenticate with a token via:
        * KUBERNETES_API_KEY: the API key to authenticate with
        * KUBERNETES_API_KEY_PREFIX: the key kind, if not set, defaults to
          "Bearer"

        Or via a username/password:
        * KUBERNETES_USERNAME
        * KUBERNETES_PASSWORD

        Or via SSL:
        * KUBERNETES_CERT_FILE
        * KUBERNETES_KEY_FILE

        Finally, you may disable SSL verification against HTTPS endpoints:
        * KUBERNETES_VERIFY_SSL: should we verify the SSL (unset means no)
        * KUBERNETES_CA_CERT_FILE: path the CA certificate when verification is
          expected
    """
    env = os.environ

    def lookup(k: str, d: str = None) -> str:
        return env.get(k, d)

    context = lookup("KUBERNETES_CONTEXT")

    try:
        config.load_kube_config(context=context)
        log.debug("Detected a local config file, context: {}".format(
            context or "default"))

        proxy_url = os.getenv('HTTP_PROXY', None)
        if proxy_url:
            client.Configuration._default.proxy = proxy_url

    except config.ConfigException as e:

        log.debug(f"Load kube config failed, error: {e}")
        log.debug("Trying incluster config...")
        try:
            config.load_incluster_config()

            proxy_url = os.getenv('HTTP_PROXY', None)
            if proxy_url:
                client.Configuration._default.proxy = proxy_url

        except config.ConfigException as e:

            log.debug(f"Incluster config failed, error: {e}")
            log.debug("Trying via Configuration obj...")

            configuration = client.Configuration()
            configuration.debug = True
            configuration.host = lookup("KUBERNETES_HOST", "http://localhost")
            configuration.verify_ssl = lookup(
                "KUBERNETES_VERIFY_SSL", False) is not False
            configuration.ssl_ca_cert = lookup("KUBERNETES_CA_CERT_FILE")

            if "KUBERNETES_API_KEY" in env:
                configuration.api_key['authorization'] = lookup(
                    "KUBERNETES_API_KEY")
                configuration.api_key_prefix['authorization'] = lookup(
                    "KUBERNETES_API_KEY_PREFIX", "Bearer")
            elif "KUBERNETES_CERT_FILE" in env:
                configuration.cert_file = lookup("KUBERNETES_CERT_FILE")
                configuration.key_file = lookup("KUBERNETES_KEY_FILE")
            elif "KUBERNETES_USERNAME" in env:
                configuration.username = lookup("KUBERNETES_USERNAME")
                configuration.password = lookup("KUBERNETES_PASSWORD", "")

            proxy_url = os.getenv('HTTP_PROXY', None)
            if proxy_url:
                configuration.proxy = proxy_url


config_client()
