import logging
from kubernetes import client, utils
from uuid import uuid1
from datetime import datetime, timedelta
import pytz


class Job:

    def __init__(self, context, logger = None, namespace='default'):
        self.client_obj = client.api_client.ApiClient()
        self.batch_v1 = client.BatchV1Api()
        self.logger = logger or logging.getLogger(__name__)
        self.namespace = namespace

    def create_job(self,
                   image,
                   command,
                   name,
                   service_account: str = None
                   ) -> str:

        try:
            id = name + '-' + str(uuid1())[:8]
            job = {
                "apiVersion": "batch/v1",
                "kind": "Job",
                "metadata": {
                    "name": id,
                    "namespace": self.namespace,
                },
                "spec": {
                    "backoffLimit": 3,
                    "template": {
                        "metadata": {
                            "labels": {
                                "app": name
                            },
                            "namespace": self.namespace
                        },
                        "spec": {
                            "containers": [
                                {
                                    "name": id,
                                    "image": image,
                                    "imagePullPolicy": "Always",
                                    "command": command,
                                    "resources": {
                                        "limits": {"memory": "999Mi"},
                                        "requests": {"memory": "999Mi"}
                                    }
                                }
                            ],
                            "restartPolicy": "Never",
                            "serviceAccountName": service_account,
                            "terminationGracePeriodSeconds": 30
                        }
                    }
                }
            }

            utils.create_from_dict(
                    self.client_obj,
                    job,
                    verbose=True,
                    namespace=self.namespace
            )
            return id

        except utils.FailToCreateError as err:
            self.logger.error(f"Error creating resource {name}: {err}")
            return None

    def get_jobs(self):
        jobs = self.batch_v1.list_namespaced_job(
            namespace=self.namespace, pretty=True).items
        return [vars(i) for i in jobs]

    def delete_job(self, name):
        return self.batch_v1.delete_namespaced_job(name, self.namespace)

    def delete_jobs_older_than(self, days: int):
        """Deletes all jobs in namespace who's start_time
        is older than now - days"""

        utc=pytz.UTC
        delta = timedelta(days = days)
        max_time = utc.localize(datetime.now() - delta)

        jobs = self.get_jobs()
        for job in jobs:
            if job['_status'].__dict__['_start_time'] < max_time:
                status = self.delete_job(job['_metadata'].__dict__['_name'])
                self.logger.info(f"Job {job['_metadata'].__dict__['_name']} deleted")

    def __getitem__(self, pos):
        return self.get_jobs()[pos]

    def __len__(self):
        return len(self.get_jobs())
